package RA;

use Salus all => 1;

header id => (
	label => 'ID',	
);

header firstName => (
	label => 'First Name',
);

header lastName => (
	label => 'Last Name',
);

header age => (
	label => 'Age',
);

1;
